/*
shows the *.FDF files in the current directory
 */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>
#include "screen.h"

/* constants for DOS file-matching routines */
#define FILESPEC        13      /* maximum filespec + NUL */
#define MAXCOL          80
#define MAXPATH		64

/* allocation quantities */
#define MAX_FILES 100

typedef char OUTA[FILESPEC];

/* package data */

static  char path[MAXPATH + 1]; /* working pathname */
static  int nfiles;		/* number of file names */
static  int ncols;		/* number of columns to output */
static  int nlines;		/* number of lines to output */
static  int mod;		/* mod columns in last row */
static  int maxlen;		/* screen columns per path name */
static  int x_off=8, y_off=5;


/* ---------------------------------------------------------------- *
   last_ch -- returns pointer to last character in string
*/
char last_ch(s)
char *s;
{
register char *cp;
    for (cp=s; *cp; ++cp);
    return (*(--cp));
}
/* end last_ch */

/* show_nth -- Display file entries
 */

static void show_nth(int x, int y, char *p)
{
        setcur(y+y_off,x*(maxlen)+x_off);
        wstr0(p);
	wchar('\2');
}

/* ---------------------------------------------------------------- *
   ls_multi -- list items in multiple columns that
   vary in width and number based on longest item size
*/

int ls_multi(buf, nelem)
char buf[MAX_FILES+1][FILESPEC];
int nelem;
{
    register int i, j;
    int errcount = 0;
    int n=0;		/* number of items in list */
    int len;
    /* get length of longest pathname and calculate number
       of columns and lines (col width = maxlen + 1)
    */
    maxlen = 0;
    while (n < nelem)
        if ((len = strlen(buf[n++])) > maxlen) maxlen = len;
    maxlen += 2;	/* Allow 2 spaces between */

    nlines = (n < 15) ? n : 15;
    ncols = n / nlines + 1;
    mod = n % nlines;
    if (!mod)		/* Don't need extra column */
	{ ncols--;
	  mod = nlines;
	}

    box(y_off-1,x_off-2,y_off+nlines,x_off+(maxlen)*ncols,2);
    setcur(y_off,x_off);

    /* output multi-column list */
    for (i = 0; i < nlines; i++)
    {   for (j = 0; j*nlines+i < n; j++)
	  show_nth(j,i,strlwr(buf[j*nlines+i]));
    }

    return (errcount);
} /* end ls_multi() */

/* ------------------------ find_x_y ----------------------------- *
*/
static int find_x_y(ix, x, y)
int ix, *x,*y;
{
	*x = ix / nlines;
	*y = ix % nlines;
}

void bubblesort(OUTA *array, int number)
{ register int i;
  register OUTA *this_el;
  int bound, nextbound, exchanges;
  OUTA temp;

	nextbound = number-1;	/* Last unsorted element */
	exchanges=1;
	while (exchanges)
	  for (i=0,exchanges=0,bound=nextbound; i<bound; i++)
	  { this_el = array+i;
	    if (strcmp((char *)this_el,(char *)(this_el+1)) > 0) /* Out of order */
	      {	strcpy(temp,(char *)this_el);		/* Swap entries */
		strcpy((char *)this_el,(char *)(this_el+1));
		strcpy((char *)(this_el+1),temp);
		exchanges++;
		nextbound = i;
	      }
	  }
}

/* ---------------------------------------------------------------- *
*/
int showfile(char *filter, int attributes)
{
    int i;
    char *pt;
    register OUTA *bp;	/* output buffer pointer */
    struct find_t entry;

	setcur(y_off,x_off);
	bp = ((OUTA *) calloc (MAX_FILES+1,FILESPEC));
	if (bp == NULL)
		return;
	memset(bp,'\0',(MAX_FILES+1) * FILESPEC);
	if(_dos_findfirst(filter,attributes,&entry))	/* Find normal files */
	{ if (attributes != _A_SUBDIR)
		wstr("\nNo files found.");
	  free(bp);
	  return;
	}
	/* add file or directory to the buffer */
	nfiles = 0;
        do
        {   if (!strcmp(entry.name,"."))
		continue;	/* Don't show '.' */
	    if (attributes == _A_SUBDIR && entry.attrib != _A_SUBDIR)
		continue;	/* Show only directories */
            strcpy(bp[nfiles],entry.name);
            ++nfiles;
        } while (_dos_findnext(&entry) == 0 && nfiles < MAX_FILES);
	if(nfiles >= MAX_FILES)
	  { wstr("More than 100 files.");
	    free(bp);
	    return;
	  }

	if (nfiles)
	  { /* sort and list */
	    bubblesort(bp,nfiles);
	    ls_multi(bp, nfiles);
	  }
	free(bp);
	return;
}
