;
;	ZCONASM.ASM -- Console driver for Zenith Z-100 (tm) Computer
;			Part 2 of 2 (See also ZCON.C)
;
;	FORM FILL-R
;
;	Copyright (c) 1989 Lindley Systems
;
;	4257 Berwick Place
;	Woodbridge, VA 22192
;	(703) 590-8890
;
;	All Rights Reserved. No part of this program may be reproduced
;	in whole or in part without express permission of Lindley Systems.
;
;


        PAGE    ,132 
        TITLE   ZCONASM.ASM - Provides Z-100 assembly interface

        public  _conout ; (TOP_ROW,BOT_ROW,NR_ROWS,DIRECTION)
        public  _scroll ; (CHARACTER)

TOP_ROW   EQU   WORD PTR [BP+6] 
LEFT_COL  EQU	WORD PTR [BP+8]
BOT_ROW   EQU   WORD PTR [BP+10] 
RIGHT_COL EQU	WORD PTR [BP+12]
NR_ROWS   EQU   WORD PTR [BP+14] 

CHARACTER EQU   WORD PTR [BP+6] 
 
CONOUT_TEXT      SEGMENT  PARA PUBLIC 'CODE' 
CONOUT_TEXT      ENDS 
SCROLL_TEXT      SEGMENT  PARA PUBLIC 'CODE' 
SCROLL_TEXT      ENDS 
_DATA   SEGMENT  WORD PUBLIC 'DATA' 
EXTRN   _color_attribs:WORD
_DATA   ENDS 
CONST   SEGMENT  WORD PUBLIC 'CONST' 
CONST   ENDS 
_BSS    SEGMENT  WORD PUBLIC 'BSS' 
_BSS    ENDS 
DGROUP  GROUP   CONST, _BSS, _DATA 
; 
 
_BIOS SEGMENT AT 00040H 
        ASSUME  CS:_BIOS
    ORG	9
BIOS_CONOUT:
_BIOS ENDS 
;
_BLUE   SEGMENT AT 0C000H   ; Blue screen segment
_BLUE   ENDS 
_RED    SEGMENT AT 0D000H   ; Red screen segment
_RED    ENDS 
_GREEN  SEGMENT AT 0E000H   ; Green screen segment
_GREEN  ENDS 
;
CONOUT_TEXT SEGMENT PARA PUBLIC 'CODE' 

_conout PROC    FAR 
        ASSUME  CS:CONOUT_TEXT
        ORG     0 
; 
        PUSH    BP              ; Save stack starting point 
        MOV     BP,SP 
; GET CONOUT PARAMETER
        MOV     AX,CHARACTER
        CALL    FAR PTR BIOS_CONOUT
        MOV     SP,BP
        POP     BP 
        RET
_conout ENDP 
; 
CONOUT_TEXT ENDS 
;
SCROLL_TEXT SEGMENT PARA PUBLIC 'CODE' 
; This routine's data is placed here to simplify routine operation.
        ASSUME  CS:SCROLL_TEXT, DS:SCROLL_TEXT, ES:SCROLL_TEXT, SS:DGROUP 
        ORG     0 

; DATA SPACE 
         DB     'SCROLL'
TOP_LINE DB     0 
BTM_LINE DB     0 
LFT_CHAR DB     0 
QTY_CHAR DB     0 
SCR_LINE DB     0 
HGT_WNDO DB     0 
CURSOR_MODE DW  0 
DAT_SEG  DW     _DATA
BLUE_SEG DW     _BLUE
RED_SEG  DW     _RED
GREEN_SEG DW    _GREEN
NORMAL   DB     0
 
;***********************  Subroutine entry point ************************ 
_scroll PROC    FAR 
; 
        PUSH    BP              ; Save stack starting point 
        MOV     BP,SP 
        PUSH    DI 
        PUSH    SI 
        PUSH    DS 
        PUSH    ES 
        MOV     AX,CS 
        MOV     DS,AX           ; Data is in the code segment 
	MOV	ES,DAT_SEG
        ASSUME  CS:SCROLL_TEXT, DS:SCROLL_TEXT, ES:_DATA, SS:DGROUP 
        MOV     AL,BYTE PTR ES:_color_attribs+2
        SHR     AL,1
        SHR     AL,1
        SHR     AL,1
        SHR     AL,1
        MOV     NORMAL,AL
        ASSUME  CS:SCROLL_TEXT, DS:SCROLL_TEXT, ES:SCROLL_TEXT, SS:DGROUP 
;       MOV     AL,008H         ; VRAM PARALLELED 
        MOV     AL,078H         ; VRAM idependently addressable
        OUT     0D8H,AL 
        CLD                     ; Be sure string instructions increment

; Get scroll parameters 
        MOV     BX,TOP_ROW
        MOV     CH,BL		; CH = Top row
	MOV	BX,LEFT_COL
        MOV     CL,BL		; CL = Left column
        MOV     BX,BOT_ROW
        MOV     DH,BL		; DH = Bottom row
	MOV	BX,RIGHT_COL
        MOV     DL,BL		; DL = Right column

; test parameters
        CMP     DH,24		; TEST BOTTOM LINE
        JBE     SCR1
        MOV     DH,24
SCR1:   MOV     BTM_LINE,DH
        CMP     CH,DH		; TOP LINE MUST BE <= BOTTOM LINE
        JBE     SCR2
        MOV     CH,DH
SCR2:   MOV     TOP_LINE,CH
        CMP     DL,79		; RIGHT CHAR
        JBE     SCR3
        MOV     DL,79
SCR3:   CMP     CL,DL		; LEFT CHAR MUST BE <= RIGHT CHAR
        JBE     SCR4
        MOV     CL,DL
SCR4:   MOV     LFT_CHAR,CL
        SUB     DL,CL
        INC     DL		; DL = Width of window (# chars)
        MOV     QTY_CHAR,DL
        SUB     DH,CH		; DH = Height of window (# lines)
        INC     DH
        MOV     HGT_WNDO,DH
; Call SCROLL_UP or SCROLL_DOWN as appropriate.
        MOV     AX,NR_ROWS
	CMP	AX,0		; Rows_to_scroll is:
	JZ	SCR4A		; =0 to Erase region
	JG	SCR5		; >0 to Scroll UP
	NEG	AX		; <0 to Scroll DOWN
	JMP	SHORT	SCR5

SCR4A:	MOV     AL,DH		; (Erase region)

SCR5:	CMP     AL,DH		; Check # of lines is <= region size;
	JBE     SCR6		;   if not,
	MOV     AL,DH		; Set # of lines to region size.

SCR6:   MOV     SCR_LINE,AL

	CMP	NR_ROWS,0	; If rows_to_scroll >=0, scroll UP.
	JGE	UP

        CALL    SCROLL_DOWN	; Else scroll DOWN.
        JMP     SHORT XIT 

UP:     CALL    SCROLL_UP 
XIT:    POP     ES 
        POP     DS 
        POP     SI
        POP     DI
        MOV     SP,BP
        POP     BP 
        RET
_scroll ENDP 
; 
SCROLL_UP PROC NEAR 
        MOV     BH,TOP_LINE 
        MOV     BL,LFT_CHAR 
        SHL     BH,1    ;enter with bh,bl = row,col of char 
        SHL     BH,1    ;exit with bx = offset 
        SHL     BH,1    ;uses bh 
        MOV     DI,BX 
        MOV     BH,TOP_LINE 
        ADD     BH,SCR_LINE 
        SHL     BH,1    ;enter with bh,bl = row,col of char 
        SHL     BH,1    ;exit with bx = offset 
        SHL     BH,1    ;uses bh 
        MOV     SI,BX 
        MOV     CL,HGT_WNDO 
        SUB     CL,SCR_LINE 
        XOR     CH,CH
        MOV     DL,QTY_CHAR 
        XOR     DH,DH 
        PUSH    DS 
        PUSH    ES 
        JCXZ    UP_BLANK
UP_AGAIN:		    ; Do one line in all three planes
        MOV	BX,CS:BLUE_SEG
        CALL    MDL             ;move a blue line 
        MOV     BX,CS:RED_SEG
        CALL    MDL             ;move a red line 
        MOV     BX,CS:GREEN_SEG
        CALL    MDL             ;move a green line 
        ADD     DI,2048
        ADD     SI,2048 
        LOOP    UP_AGAIN
UP_BLANK:
        MOV     AX,CS
        MOV     DS,AX		; Point to our data
        XOR     CH,CH 
        MOV     CL,SCR_LINE	; Lines to blank out
        JCXZ    UP_B2 
; Loop to blank lines to normal background color
UP_B1:  PUSH    CX
	MOV	BX,BLUE_SEG
        XOR     CH,CH
        TEST    CS:NORMAL,1	; 0 = BLANK, 1 = ON
	JZ      UP_NB2
	DEC     CH
UP_NB2: CALL    BDL		; blank a blue line
        MOV     BX,RED_SEG
        XOR     CH,CH
        TEST    CS:NORMAL,2	; 0 = BLANK, 1 = ON
	JZ      UP_NR2
	DEC     CH
UP_NR2: CALL    BDL		; blank a red line
        MOV     BX,GREEN_SEG
        XOR     CH,CH
        TEST    CS:NORMAL,4	; 0 = BLANK, 1 = ON
	JZ      UP_NG2
	DEC     CH
UP_NG2: CALL    BDL		; blank a green line
        ADD     DI,2048 
        POP     CX
        LOOP    UP_B1 
UP_B2:	POP     ES 
        POP     DS 
        RET 
SCROLL_UP ENDP 
;
; 
SCROLL_DOWN PROC NEAR 
        MOV     BH,BTM_LINE 
        MOV     BL,LFT_CHAR 
        SHL     BH,1    ;Destination line
        SHL     BH,1
        SHL     BH,1
        MOV     DI,BX 
        MOV     BH,BTM_LINE 
        SUB     BH,SCR_LINE 
        SHL     BH,1    ;Source line 
        SHL     BH,1
        SHL     BH,1
        MOV     SI,BX 
        MOV     CL,HGT_WNDO 
        SUB     CL,SCR_LINE
	XOR     CH,CH		; Lines to move
        MOV     DL,QTY_CHAR 
        XOR     DH,DH           ; Bytes on line to move
        PUSH    DS
        PUSH    ES
        JCXZ    DN_BLANK
DN_AGAIN:		    ; Do one line in all three planes
        MOV	BX,CS:BLUE_SEG
        CALL    MDL             ;move a blue line 
        MOV     BX,CS:RED_SEG
        CALL    MDL             ;move a red line 
        MOV     BX,CS:GREEN_SEG
        CALL    MDL             ;move a green line 
        SUB     DI,2048
        SUB     SI,2048 
        LOOP    DN_AGAIN
DN_BLANK:
        MOV     AX,CS
        MOV     DS,AX		; Point to our data
        XOR     CH,CH 
        MOV     CL,SCR_LINE	; Lines to blank out
        JCXZ    DN_B2 
; Loop to blank lines to normal background color
DN_B1:  PUSH    CX
	MOV	BX,BLUE_SEG
        XOR     CH,CH
        TEST    CS:NORMAL,1	; 0 = BLANK, 1 = ON
	JZ      DN_NB2
	DEC     CH
DN_NB2: CALL    BDL		; blank a blue line
        MOV     BX,RED_SEG
        XOR     CH,CH
        TEST    CS:NORMAL,2	; 0 = BLANK, 1 = ON
	JZ      DN_NR2
	DEC     CH
DN_NR2: CALL    BDL		; blank a red line
        MOV     BX,GREEN_SEG
        XOR     CH,CH
        TEST    CS:NORMAL,4	; 0 = BLANK, 1 = ON
	JZ      DN_NG2
	DEC     CH
DN_NG2: CALL    BDL		; blank a green line
        SUB     DI,2048 
        POP     CX
        LOOP    DN_B1 
DN_B2:	POP     ES 
        POP     DS 
        RET 
SCROLL_DOWN ENDP 
; 
; MOVE ONE ROW OF VIDEO, CHARACTERS, AND ATTRIBUTES 
; BX = Color plane segment
; SI = Source offset
; DI = Destination offset
; DX = Number of bytes to move
;
MDL     PROC    NEAR 
        MOV     DS,BX
        MOV     ES,BX
        PUSH    AX 
        PUSH    CX
        PUSH    DI
        PUSH    SI
        MOV     CX,11
        CLD
MDL1:   PUSH    CX
        PUSH    DI
        PUSH    SI
        MOV     CX,DX
        XOR     CH,CH
        REP MOVSB
        POP     SI 
        POP     DI 
        POP     CX 
        ADD     DI,128 
        ADD     SI,128 
        LOOP    MDL1 
        POP     SI 
        POP     DI 
        POP     CX
        POP     AX 
        RET 
MDL     ENDP  
;
BDL PROC NEAR		; BLANK A DISPLAY LINE 
; BX = Color plane segment
; DI = Destination offset
; DX = Number of bytes to move
; CH = Plane fill data
;
        PUSH    AX
        PUSH    DI 
        PUSH    CX
        MOV     ES,BX
        CLD             ; enter with same register set-up as 
        MOV     AL,CH   ; Fill data
        MOV     CX,11 
BDL1:   PUSH    CX      ;  above except that si is not used 
        PUSH    DI 
        MOV     CX,DX   ;  DX = no. of chars per line 
        XOR     CH,CH 
        REP     STOSB 
        POP     DI 
        POP     CX 
        ADD     DI,128 
        LOOP    BDL1 
        POP     CX
        POP     DI 
        POP     AX
        RET 
BDL     ENDP 
;
SCROLL_TEXT ENDS 
        END 
